/////////////////////////////////////////////////////////////////////////////
// CCom: class for management of the serial port
//
// With the exception of IsOpen,CCom and ~CCom every function here returns 
// an error code in the case of failure and <0> in case of success
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#include <string.h>

#include "Com.h"

/////////////////////////////////////////////////////////////////////////////
// CCom: contructor

CCom::CCom(void)
{
	cDuplex=0;
	cMessageOut=0;

	m_cOpen=0;
	hCom=NULL;
}

/////////////////////////////////////////////////////////////////////////////
// ~CCom: destructor for CCom

CCom::~CCom(void)
{
	if(m_cOpen!=0)
	{
		Close();
	}
}

/////////////////////////////////////////////////////////////////////////////
// Open: gain exclusive access to one serial port

DWORD CCom::Open(char ComPort)
{
	char port[16];
	sprintf(port,"COM%d",ComPort);
	hCom=CreateFile(port,GENERIC_READ | GENERIC_WRITE,
			        0,                      // must be 0 for communication device (exclusive access) 
			        NULL,                   // otherwise SECURITY_ATTRIBUTES structure must be set
			        OPEN_EXISTING,          // must be OPEN_EXISTING for communication device
			        FILE_FLAG_NO_BUFFERING, // may test other flags
			        NULL);                  // must be NULL under Win95/98
	if(hCom==INVALID_HANDLE_VALUE)
	{
		m_cOpen=0;
		return GetLastError();
	}
	else
	{
		m_cOpen=1;
		return 0;
	}
}

/////////////////////////////////////////////////////////////////////////////
// Close: free the serial port 

DWORD CCom::Close(void)
{
	char et; // errortest
	if(m_cOpen!=0)
	{
		et=CloseHandle(hCom);
	}
	if(et!=0)
	{
		m_cOpen=0;
		return 0;
	}
	else
	{
		m_cOpen=1;
		return GetLastError();
	}
}

/////////////////////////////////////////////////////////////////////////////
// GetDCB: fill the DCB structure with valid variables

DWORD CCom::GetDCB(DCB* ComState)
{
	char et=1; // errortest
	if(m_cOpen!=0)
	{
		et=GetCommState(hCom,ComState);
	}
	if(et!=0)
		return 0;
	else
	{
		return GetLastError();
	}
}

/////////////////////////////////////////////////////////////////////////////
// SetDCB: update CommState with (modified) DCB structure 

DWORD CCom::SetDCB(DCB* ComState)
{
	char et=1; // errortest
	if(m_cOpen!=0)
	{
		et=SetCommState(hCom,ComState);
	}
	if(et!=0)
		return 0;
	else
	{
		return GetLastError();
	}
}

/////////////////////////////////////////////////////////////////////////////
// GetTimeouts: get current timeouts ;-)

DWORD CCom::GetTimeouts(COMMTIMEOUTS* Timeouts)
{
	char et=1; // errortest
	if(m_cOpen!=0)
	{
		et=GetCommTimeouts(hCom,Timeouts);
	}
	if(et!=0)
		return 0;
	else
	{
		return GetLastError();
	}
}

/////////////////////////////////////////////////////////////////////////////
// SetTimeouts: set timeouts to values defined in the Timeouts structure

DWORD CCom::SetTimeouts(COMMTIMEOUTS* Timeouts)
{
	char et=1; // errortest
	if(m_cOpen!=0)
	{
		et=SetCommTimeouts(hCom,Timeouts);
	}
	if(et!=0)
		return 0;
	else
	{
		return GetLastError();
	}
}

/////////////////////////////////////////////////////////////////////////////
// OverWriteState: initialize the serial port with the SCom structure IniCom 

DWORD CCom::OverwriteState(SCom* IniCom)
{
	DWORD res; // errorcode
	DCB  ComState;
	COMMTIMEOUTS Timeouts;

	res=GetDCB(&ComState);
	if(res!=0)
		return res;

	res=GetTimeouts(&Timeouts);
	if(res!=0)
		return res;

	ComState.DCBlength=sizeof(DCB);
	ComState.BaudRate=IniCom->BaudRate;
	ComState.ByteSize=IniCom->ByteSize;
	ComState.Parity=IniCom->Parity;
	ComState.StopBits=IniCom->StopBits;
	// the 'standard' values are system default - not to be used
	// -> the 'useful' values are the ones tested with the NG360 inclinometer
	Timeouts.ReadIntervalTimeout=IniCom->ReadIntervalTimeout;                 // standard: 0 useful: 200, 500
	Timeouts.ReadTotalTimeoutMultiplier=IniCom->ReadTotalTimeoutMultiplier;   // standard: 0 useful: 7
	Timeouts.ReadTotalTimeoutConstant=IniCom->ReadTotalTimeoutConstant;       // standard: 0 useful: 124
	Timeouts.WriteTotalTimeoutMultiplier=IniCom->WriteTotalTimeoutMultiplier; // standard: 0 useful: 0
	Timeouts.WriteTotalTimeoutConstant=IniCom->WriteTotalTimeoutConstant;     // standard: 0 useful: 0
	if(cDuplex==1) 
	{
		ComState.fDtrControl=DTR_CONTROL_DISABLE;
		ComState.fRtsControl=RTS_CONTROL_DISABLE;
		ComState.fOutxDsrFlow=FALSE;
	}

	res=SetDCB(&ComState);
	if(res!=0)
		return res;

	res=SetTimeouts(&Timeouts);
	if(res!=0)
		return res;

	return 0;
}

/////////////////////////////////////////////////////////////////////////////
// Write: output data through port and give the user feedback in case of 
// incomplete transmission 

DWORD CCom::Write(const char* data)
{
	char et; // errortest
	unsigned long byteswritten;

	if(cDuplex==1) 
	{
		EscapeCommFunction(hCom,CLRDTR); // change voltage on the appropriate serial wires to 
		EscapeCommFunction(hCom,SETRTS); // change from read mode to write mode
	}
	et=WriteFile(hCom,data,strlen(data),&byteswritten,NULL);
	if(et!=0)
	{
		if(byteswritten!=strlen(data)) 
		{
			if(cMessageOut)
				MessageBox(NULL,"CCom.Write: Not all data could be send.","COM Error",MB_OK | MB_ICONINFORMATION);
			return 1121; // ERROR_COUNTER_TIMEOUT 
		} else {
			return 0;
		}
	}
	else
	{
		return GetLastError();
	}
}

/////////////////////////////////////////////////////////////////////////////
// Read: input data through port 

DWORD CCom::Read(char* data,unsigned long size)
{
	char et; // errortest
	unsigned long bytesread;
	DWORD ce=0;

	if(cDuplex==1) 
	{
		EscapeCommFunction(hCom,CLRRTS); // send signal to change from ready to send
		EscapeCommFunction(hCom,SETDTR); // to data terminal ready mode
	}
	et=ReadFile(hCom,data,size,&bytesread,NULL); // size: maximum number of bytes to read
		        
	data[bytesread]=0;
	data[bytesread+1]=0;

	if(et!=0)
	{
		return 0;
	}
	else
	{
		return GetLastError();
	}
}

/////////////////////////////////////////////////////////////////////////////
// IsOpen: returns <0> for closed port / <1> for open port

char CCom::IsOpen(void)
{
	return m_cOpen;
} 
